@@ -1,9 +1,9 @@
+/* gpsd.h -- fundamental types and structures for the gpsd library */
 /* This file is generated.  Do not hand-hack it! */
 
-#ifndef _gpsd_h_
-#define _gpsd_h_
-
-/* gpsd.h -- fundamental types and structures for the gpsd library */
+/* $Id: gpsd.h-head 4794 2008-08-03 16:42:57Z ckuethe $ */
+#ifndef _GPSD_H_
+#define _GPSD_H_
 
 #include <stdbool.h>
 #include <stdio.h>
@@ -17,7 +17,8 @@
 
 /* Feature configuration switches begin here */
 #define PACKAGE_VERSION ""
-#define VERSION "2.37"
+#define VERSION "2.39"
+#define AIVDM_ENABLE 1
 #define ASHTECH_ENABLE 1
 #define DBUS_ENABLE 1
 #define EARTHMATE_ENABLE 1
@@ -27,21 +28,25 @@
 #define GARMIN_ENABLE 1
 #define GPSCLOCK_ENABLE 1
 #define ITRAX_ENABLE 1
+#define MKT3301_ENABLE 1
 #define NAVCOM_ENABLE 1
 #define NMEA_ENABLE 1
 #define NTPSHM_ENABLE 1
 #define NTRIP_ENABLE 1
+/* #undef OCEANSERVER_ENABLE */
 #define PPS_ENABLE 1
-#define RTCM104_ENABLE 1
+#define RTCM104V2_ENABLE 1
+#define RTCM104V3_ENABLE 1
 #define SIRF_ENABLE 1
 #define SQUELCH_ENABLE 1
+#define SUPERSTAR2_ENABLE 1
 #define TNT_ENABLE 1
 #define TRIPMATE_ENABLE 1
 #define TSIP_ENABLE 1
 #define UBX_ENABLE 1
 /* Feature configuration switches end here */
 
-/* $Id: gpsd.h-tail 4665 2008-01-21 00:43:01Z garyemiller $ */
+/* $Id: gpsd.h-tail 5498 2009-03-18 17:37:53Z esr $ */
 
 /* Some internal capabilities depend on which drivers we're compiling. */
 #ifdef EARTHMATE_ENABLE
@@ -66,7 +71,7 @@ enum isgpsstat_t {
 };
 #define ISGPS_ERRLEVEL_BASE	5
 
-#define RTCM_MAX	(RTCM_WORDS_MAX * sizeof(isgps30bits_t))
+#define RTCM_MAX	(RTCM2_WORDS_MAX * sizeof(isgps30bits_t))
 
 /*
  * The packet buffers need to be as long than the longest packet we
@@ -92,7 +97,8 @@ enum isgpsstat_t {
 struct gps_packet_t {
     /* packet-getter internals */
     int	type;
-#define BAD_PACKET	-1
+#define BAD_PACKET	-2
+#define EMPTY_PACKET	-1
 #define COMMENT_PACKET	0
 #define NMEA_PACKET	1
 #define SIRF_PACKET	2
@@ -100,11 +106,15 @@ struct gps_packet_t {
 #define TSIP_PACKET	4
 #define EVERMORE_PACKET	5
 #define ITALK_PACKET	6
-#define RTCM_PACKET	7
-#define GARMIN_PACKET	8
-#define NAVCOM_PACKET	9
-#define UBX_PACKET	10
-#define GARMINTXT_PACKET	11
+#define GARMIN_PACKET	7
+#define NAVCOM_PACKET	8
+#define RTCM2_PACKET	9
+#define RTCM3_PACKET	10
+#define UBX_PACKET	11
+#define GARMINTXT_PACKET	12
+#define SUPERSTAR2_PACKET	13
+#define AIVDM_PACKET	14
+#define TEXTUAL_PACKET_TYPE(n)	((n)==NMEA_PACKET||(n)==GARMINTXT_PACKET||(n)==COMMENT_PACKET||(n)==AIVDM_PACKET)
     unsigned int state;
     size_t length;
     unsigned char inbuffer[MAX_PACKET_LENGTH*2+1];
@@ -118,7 +128,7 @@ struct gps_packet_t {
     unsigned counter;			/* packets since last driver switch */
     /*
      * This is not conditionalized on RTCM104_ENABLE because we need to
-     * be able to build rtcmdecode even when RTCM support is not
+     * be able to build gpsdecode even when RTCM support is not
      * configured in the daemon.
      */
     struct {
@@ -126,16 +136,17 @@ struct gps_packet_t {
 	bool            locked;
 	int             curr_offset;
 	isgps30bits_t   curr_word;
-	isgps30bits_t   buf[RTCM_WORDS_MAX];
+	isgps30bits_t   buf[RTCM2_WORDS_MAX];
 	unsigned int    bufindex;
     } isgps;
 };
 
 extern void packet_reset(struct gps_packet_t *);
 extern void packet_pushback(struct gps_packet_t *);
-extern ssize_t packet_parse(struct gps_packet_t *, size_t);
+extern void packet_parse(struct gps_packet_t *);
 extern ssize_t packet_get(int, struct gps_packet_t *);
 extern int packet_sniff(struct gps_packet_t *);
+#define packet_buffered_input(lexer) ((lexer)->inbuffer + (lexer)->inbuflen - (lexer)->inbufptr)
 
 extern void isgps_init(/*@out@*/struct gps_packet_t *);
 enum isgpsstat_t isgps_decode(struct gps_packet_t *, 
@@ -146,12 +157,15 @@ enum isgpsstat_t isgps_decode(struct gps_packet_t *,
 extern unsigned int isgps_parity(isgps30bits_t);
 extern void isgps_output_magnavox(isgps30bits_t *, unsigned int, FILE *);
 
-extern enum isgpsstat_t rtcm_decode(struct gps_packet_t *, unsigned int);
-extern void rtcm_dump(struct rtcm_t *, /*@out@*/char[], size_t);
-extern int rtcm_undump(/*@out@*/struct rtcm_t *, char *);
-extern void rtcm_unpack(/*@out@*/struct rtcm_t *, char *);
-extern bool rtcm_repack(struct rtcm_t *, isgps30bits_t *);
-extern void rtcm_output_magnavox(isgps30bits_t *, FILE *);
+extern enum isgpsstat_t rtcm2_decode(struct gps_packet_t *, unsigned int);
+extern void rtcm2_dump(struct rtcm2_t *, /*@out@*/char[], size_t);
+extern int rtcm2_undump(/*@out@*/struct rtcm2_t *, char *);
+extern void rtcm2_unpack(/*@out@*/struct rtcm2_t *, char *);
+extern bool rtcm2_repack(struct rtcm2_t *, isgps30bits_t *);
+extern void rtcm2_output_magnavox(isgps30bits_t *, FILE *);
+
+extern void rtcm3_unpack(/*@out@*/struct rtcm3_t *, char *);
+extern void rtcm3_dump(struct rtcm3_t *rtcm, FILE *);
 
 /* Next, declarations for the core library... */
 
@@ -202,6 +216,16 @@ struct gps_context_t {
 #endif /* NTPSHM_ENABLE */
 };
 
+struct aivdm_context_t {
+    /* hold context for decoding AIDVM packet sequences */
+    int part, await;		/* for tracking AIDVM parts in a multipart sequence */
+    unsigned char *field[NMEA_MAX];
+    unsigned char fieldcopy[NMEA_MAX+1];
+    unsigned char bits[2048];
+    size_t bitlen;
+    struct ais_t decoded;
+};
+
 struct gps_device_t;
 
 #if defined (HAVE_SYS_TERMIOS_H)
@@ -212,35 +236,40 @@ struct gps_device_t;
 #endif
 #endif
 
+#define MODE_NMEA	0
+#define MODE_BINARY	1
+
 struct gps_type_t {
 /* GPS method table, describes how to talk to a particular GPS type */
     /*@observer@*/char *type_name;
+    int packet_type;
     /*@observer@*//*@null@*/char *trigger;
     int channels;
     /*@null@*/bool (*probe_detect)(struct gps_device_t *session);
     /*@null@*/void (*probe_wakeup)(struct gps_device_t *session);
     /*@null@*/void (*probe_subtype)(struct gps_device_t *session, unsigned int seq);
-#ifdef ALLOW_RECONFIGURE 
-    /*@null@*/void (*configurator)(struct gps_device_t *session, unsigned int seq);
-#endif /* ALLOW_RECONFIGURE */
     /*@null@*/ssize_t (*get_packet)(struct gps_device_t *session);
     /*@null@*/gps_mask_t (*parse_packet)(struct gps_device_t *session);
     /*@null@*/ssize_t (*rtcm_writer)(struct gps_device_t *session, char *rtcmbuf, size_t rtcmbytes);
-    /*@null@*/bool (*speed_switcher)(struct gps_device_t *session, speed_t speed);
+#ifdef ALLOW_CONTROLSEND
+    /*@null@*/ssize_t (*control_send)(struct gps_device_t *session, char *buf, size_t buflen);
+#endif /* ALLOW_CONTROLSEND */
+#ifdef ALLOW_RECONFIGURE 
+    /*@null@*/void (*configurator)(struct gps_device_t *session, unsigned int seq);
+    /*@null@*/bool (*speed_switcher)(struct gps_device_t *session, 
+				     speed_t speed, char parity, int stopbits);
     /*@null@*/void (*mode_switcher)(struct gps_device_t *session, int mode);
     /*@null@*/bool (*rate_switcher)(struct gps_device_t *session, double rate);
-    int cycle_chars;
-#ifdef ALLOW_RECONFIGURE 
+    double min_cycle;
     /*@null@*/void (*revert)(struct gps_device_t *session);
 #endif /* ALLOW_RECONFIGURE */
     /*@null@*/void (*wrapup)(struct gps_device_t *session);
-    double cycle;
 };
 
 struct gps_device_t {
 /* session object, encapsulates all global state */
     struct gps_data_t gpsdata;
-    /*@relnull@*/struct gps_type_t *device_type;
+    /*@relnull@*/const struct gps_type_t *device_type;
     struct gps_context_t	*context;
 #ifdef ALLOW_RECONFIGURE
     bool enable_reconfigure;		/* OK to hack GPS settings? */ 
@@ -259,11 +288,10 @@ struct gps_device_t {
     int shmTimeP;
 # endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
-    double mag_var;		/* Magnetic variation in degrees */  
-    bool back_to_nmea;		/* back to NMEA on revert? */
-#ifdef NMEADISC
-    int ldisc;			/* current line discipline */
-#endif
+    double mag_var;			/* Magnetic variation in degrees */  
+    bool back_to_nmea;			/* back to NMEA on revert? */
+    char msgbuf[MAX_PACKET_LENGTH*2+1];	/* command message buffer for sends */
+    size_t msgbuflen;
     /*
      * The rest of this structure is driver-specific private storage.
      * Because the Garmin driver uses a long buffer, you can have
@@ -276,11 +304,16 @@ struct gps_device_t {
 	    int part, await;		/* for tracking GSV parts */
 	    struct tm date;
 	    double subseconds;
+	    char *field[NMEA_MAX];
+	    unsigned char fieldcopy[NMEA_MAX+1];
 #ifdef GPSCLOCK_ENABLE
 	    bool ignore_trailing_edge;
 #endif /* GPSCLOCK_ENABLE */
 	} nmea;
 #endif /* NMEA_ENABLE */
+#ifdef AIVDM_ENABLE
+	struct aivdm_context_t aivdm;
+#endif /* AIVDM_ENABLE */
 #ifdef BINARY_ENABLE
 #ifdef SIRF_ENABLE
 	struct {
@@ -308,6 +341,11 @@ struct gps_device_t {
 #endif /* ALLOW_RECONFIGURE */
 	} sirf;
 #endif /* SIRF_ENABLE */
+#ifdef SUPERSTAR2_ENABLE
+	struct {
+	    unsigned short gps_week;
+	} superstar2;
+#endif /* SUPERSTAR2_ENABLE */
 #ifdef TSIP_ENABLE
 	struct {
 	    int16_t gps_week;		/* Current GPS week number */
@@ -351,7 +389,7 @@ struct gps_device_t {
 #endif /* NAVCOM_ENABLE */
 	/*
 	 * This is not conditionalized on RTCM104_ENABLE because we need to
-	 * be able to build rtcmdecode even when RTCM support is not
+	 * be able to build gpsdecode even when RTCM support is not
 	 * configured in the daemon.  It doesn't take up extra space.
 	 */
 	struct {
@@ -359,7 +397,7 @@ struct gps_device_t {
 	    bool            locked;
 	    int             curr_offset;
 	    isgps30bits_t   curr_word;
-	    isgps30bits_t   buf[RTCM_WORDS_MAX];
+	    isgps30bits_t   buf[RTCM2_WORDS_MAX];
 	    unsigned int    bufindex;
 	} isgps;
 #endif /* BINARY_ENABLE */
@@ -378,12 +416,13 @@ struct gps_device_t {
 #define IS_HIGHEST_BIT(v,m)	(v & ~((m<<1)-1))==0
 
 /* here are the available GPS drivers */
-extern struct gps_type_t **gpsd_drivers;
+extern const struct gps_type_t **gpsd_drivers;
 
 /* gpsd library internal prototypes */
 extern gps_mask_t nmea_parse_input(struct gps_device_t *);
 extern gps_mask_t nmea_parse(char *, struct gps_device_t *);
-extern int nmea_send(int, const char *, ... );
+extern ssize_t nmea_write(struct gps_device_t *session, char *buf, size_t len);
+extern ssize_t nmea_send(struct gps_device_t *session, const char *, ... );
 extern void nmea_add_checksum(char *);
 
 ssize_t generic_get(struct gps_device_t *);
@@ -394,6 +433,7 @@ extern gps_mask_t evermore_parse(struct gps_device_t *, unsigned char *, size_t)
 extern gps_mask_t navcom_parse(struct gps_device_t *, unsigned char *, size_t);
 extern gps_mask_t garmin_ser_parse(struct gps_device_t *);
 extern gps_mask_t garmintxt_parse(struct gps_device_t *);
+extern gps_mask_t aivdm_parse(struct gps_device_t *);
 
 extern bool dgnss_url(char *);
 extern int dgnss_open(struct gps_context_t *, char *);
@@ -402,7 +442,7 @@ extern void dgnss_report(struct gps_device_t *);
 extern void dgnss_autoconnect(struct gps_context_t *, double, double);
 
 extern void rtcm_relay(struct gps_device_t *);
-extern void rtcm_output_mag(isgps30bits_t *, FILE *);
+extern void rtcm2_output_mag(isgps30bits_t *, FILE *);
 
 extern int dgpsip_open(struct gps_context_t *, const char *);
 extern void dgpsip_report(struct gps_device_t *);
@@ -424,9 +464,12 @@ extern void gpsd_close(struct gps_device_t *);
 
 extern void gpsd_zero_satellites(/*@out@*/struct gps_data_t *sp)/*@modifies sp@*/;
 extern void gpsd_interpret_subframe(struct gps_device_t *, unsigned int[]);
-extern /*@ observer @*/ char *gpsd_hexdump(const void *, size_t);
-extern int gpsd_hexpack(char *, char *, int);
+extern int gpsd_hexdump_level;
+extern /*@ observer @*/ char *gpsd_hexdump(/*@null@*/const void *, size_t);
+extern /*@ observer @*/ char *gpsd_hexdump_wrapper(/*@null@*/const void *, size_t, int);
+extern int gpsd_hexpack(/*@in@*/char *, /*@out@*/char *, size_t);
 extern int hex2bin(char *);
+extern ssize_t hex_escapes(/*@out@*/char *cooked, const char *raw);
 extern void ntpd_link_activate(struct gps_device_t *session);
 extern char /*@observer@*/ *gpsd_id(/*@in@*/struct gps_device_t *);
 extern void gpsd_position_fix_dump(struct gps_device_t *, /*@out@*/char[], size_t);
@@ -458,18 +501,39 @@ extern unsigned char hc(unsigned char);
 bool sirf_write(int fd, unsigned char *msg);
 
 /* application interface */
-extern void gpsd_init(struct gps_device_t *, struct gps_context_t *, char *);
+extern void gpsd_init(struct gps_device_t *, 
+		      struct gps_context_t *, 
+		      /*@null@*/char *);
 extern int gpsd_activate(struct gps_device_t *, bool);
 extern void gpsd_deactivate(struct gps_device_t *);
 extern gps_mask_t gpsd_poll(struct gps_device_t *);
 extern void gpsd_wrap(struct gps_device_t *);
 
+/* exceptional driver methods */
+#ifdef UBX_ENABLE
+extern gps_mask_t ubx_parse(struct gps_device_t *, unsigned char *, size_t);
+extern bool ubx_write(struct gps_device_t *, unsigned int, unsigned int, 
+		      /*@null@*/unsigned char *, unsigned short);
+extern void ubx_catch_model(struct gps_device_t *,
+			    unsigned char *, size_t);
+#endif /* UBX_ENABLE */
+#ifdef AIVDM_ENABLE
+extern bool aivdm_decode(char *, size_t, struct aivdm_context_t *);
+extern void  aivdm_dump(struct ais_t *, bool, bool, FILE *);
+#endif /* AIVDM_ENABLE */
+#ifdef MKT3301_ENABLE
+    extern gps_mask_t processMKT3301(int c UNUSED, char *field[], struct gps_device_t *session);
+#endif /* MKT3301_ENABLE */
+
 /* caller should supply this */
+# if __GNUC__ >= 3 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
+__attribute__((__format__(__printf__, 2, 3))) void gpsd_report(int, const char *, ...);
+# else /* not a new enough GCC, use the unprotected prototype */
 void gpsd_report(int, const char *, ...);
+#endif
 
 #ifdef S_SPLINT_S
 extern bool finite(double);
-extern int cfmakeraw(struct termios *);
 extern struct protoent *getprotobyname(const char *);
 extern /*@observer@*/char *strptime(const char *,const char *tp,/*@out@*/struct tm *)/*@modifies tp@*/;
 extern struct tm *gmtime_r(const time_t *,/*@out@*/struct tm *tp)/*@modifies tp@*/;
@@ -496,8 +560,7 @@ extern float roundf(float x);
 void cfmakeraw(struct termios *);
 #endif /* defined(__CYGWIN__) */
 
-#endif /* _gpsd_h_ */
-
+#endif /* _GPSD_H_ */
 // Local variables:
 // mode: c
 // end:
